## Sid Meier's Civilization 4
## Copyright Firaxis Games 2005
## 
## Author 	: 12monkeys
## Project	: Raze City Mod
## Date 	: 27.04.2006
## version 	: 1.0
## 

from CvPythonExtensions import *
import CvUtil
import CvConfigParser
import MonkeyTools as mt
import SdToolKitAdvanced as sd

gc = CyGlobalContext()
localText = CyTranslator()
cg = CyGame()

RAZEMODE_NONE = 0
RAZEMODE_ACTIVE = 1
RAZEMODE_PAUSE = 2

# --------------------------------------------------------------------------
# function returns true if the population of the city 
# is lower or equal to the configured value. If so
# the city will be destroyed
def isPopLimitReached(pCity):
	return pCity.getPopulation() <= getPopLimit()

# --------------------------------------------------------------------------
# reads out the population limit from the ini file.
def getPopLimit():
	config = CvConfigParser.CvConfigParser("RazeCityMod.ini")
	if (config != None):
		return config.getint( "General", "Raze City Pop Limit", 3 )
	else:
		return 3

# --------------------------------------------------------------------------
# reads out the population decrement from the ini file.
def getPopDecr():
	config = CvConfigParser.CvConfigParser("RazeCityMod.ini")
	if (config != None):
		return config.getint( "General", "Raze City Pop Decrement", 2 )
	else:
		return 2

# --------------------------------------------------------------------------
# reads out the building decrement from the ini file.
def getBuildDecr():
	config = CvConfigParser.CvConfigParser("RazeCityMod.ini")
	if (config != None):
		return config.getint( "General", "Raze City Building Decrement", 2 )
	else:
		return 2

# --------------------------------------------------------------------------
# reads out the building decrement from the ini file.
def getNameSuffixActive():
	config = CvConfigParser.CvConfigParser("RazeCityMod.ini")
	if (config != None):
		return config.get( "General", "Raze City Name Suffix Active", "!R!" )
	else:
		return "!R!"

# --------------------------------------------------------------------------
# reads out the building decrement from the ini file.
def getNameSuffixPause():
	config = CvConfigParser.CvConfigParser("RazeCityMod.ini")
	if (config != None):
		return config.get( "General", "Raze City Name Suffix Pause", "?R?" )
	else:
		return "?R?"

# --------------------------------------------------------------------------
# function checks if there is a ground combat unit in the city
def checkForGroundCombatUnit(pCity):
	iOwner = pCity.getOwner()
	pPlot = pCity.plot()
	for i in range(pPlot.getNumUnits()):
		pUnit = pPlot.getUnit(i)
		if pUnit.baseCombatStr() > 0:
			if pUnit.getDomainType() == DomainTypes.DOMAIN_LAND:
				if pUnit.getOwner() == iOwner:
					return true
	return false

# --------------------------------------------------------------------------
# function sets the cities raze mode (paused, active or none)
def setCityRazeMode(pCity, nMode):
	if not sd.sdObjectExists( "RazeCityMod", pCity ):
		sd.sdObjectInit("RazeCityMod", pCity, {"RazeFlag":RAZEMODE_NONE})
	sd.sdObjectSetVal("RazeCityMod", pCity, "RazeFlag", nMode)
	setCityName(pCity, nMode)
	
# --------------------------------------------------------------------------
# function returns the cities raze mode (paused, active or none)
def getCityRazeMode(pCity):
	if sd.sdObjectExists( "RazeCityMod", pCity ):
		return sd.sdObjectGetVal("RazeCityMod", pCity, "RazeFlag")
	else:
		return RAZEMODE_NONE

# --------------------------------------------------------------------------
# function adds a suffix to the cities name according to the current raze mode
def setCityName(pCity, nMode):
	# inits
	sSuffixPause = getNameSuffixPause()
	sSuffixActive = getNameSuffixActive()
	sActName = pCity.getName()
	# check for mode
	if nMode == RAZEMODE_NONE:
		# removes all existing suffixes
		if sActName.find(sSuffixActive) == (len(sActName)-len(sSuffixActive)):
			sActName = sActName.replace(sSuffixActive, "")
			pCity.setName(sActName, false)
		elif sActName.find(sSuffixPause) == (len(sActName)-len(sSuffixPause)):
			sActName = sActName.replace(sSuffixPause, "")
			pCity.setName(sActName, false)
	elif nMode == RAZEMODE_ACTIVE:
		if len(sActName)+len(sSuffixPause) <= 30:
			if sActName.find(sSuffixPause) == (len(sActName)-len(sSuffixPause)):
				sActName = sActName.replace(sSuffixPause, "")
				pCity.setName(sActName, false)
			if sActName.find(sSuffixActive) != (len(sActName)-len(sSuffixActive)):
				sActName = (sActName + sSuffixActive)
				pCity.setName(sActName, false)
	elif nMode == RAZEMODE_PAUSE:
		if len(sActName)+len(sSuffixPause) <= 30:
			if sActName.find(sSuffixActive) == (len(sActName)-len(sSuffixActive)):
				sActName = sActName.replace(sSuffixActive, "")
				pCity.setName(sActName, false)
			if sActName.find(sSuffixPause) != (len(sActName)-len(sSuffixPause)):
				sActName = (sActName + sSuffixPause)
				pCity.setName(sActName, false)
		
# --------------------------------------------------------------------------
# performs the razing process per turn
# Function returns TRUE is razing mode is still active and FALSE if not
def doRazingTurn(pCity):
	
	# some inits
	iOwner = pCity.getOwner()
	pPlot = pCity.plot()
	iPop = pCity.getPopulation()
	iPopDecr = getPopDecr()
	iBuildDecr = getBuildDecr()

	# checks if the population limit is reached.
	if isPopLimitReached(pCity) or (iPop <= iPopDecr):
		sd.sdObjectWipe( "RazeCityMod", pCity )
		pCity.doTask(TaskTypes.TASK_RAZE, 0, 0, false)
		return false
	
	# adjusts occupation timer (= rounds the occupation status persists) 
	# Is used for tha case the razing ends for any reason
	pCity.setOccupationTimer(iPop)

	# check if there is a ground combat unit in the city
	if not checkForGroundCombatUnit(pCity): 
		# no military unit found -> pause razing
		setCityRazeMode(pCity, RAZEMODE_PAUSE)
		return false
		
	# reducing city population
	if (iPop > iPopDecr):
		pCity.setPopulation(iPop-iPopDecr)
				
	# create a list with the cities building
	lBuildings = []
	for i in range(gc.getNumBuildingInfos()):
		if (pCity.hasBuilding(i)):
			lBuildings.append(i)
			
	# destroy a random building(s)
	# destroys only buildings which are not prerequisites for other existing buildings in the city.
	for n in range(getBuildDecr()):
		bFound = false
		while (not bFound) and (len(lBuildings) != 0):
			iRandomBuilding = lBuildings[cg.getSorenRandNum(len(lBuildings), "RCM")]
			bNeeded = false
			for i in range(len(lBuildings)):
				iLoopBuilding = lBuildings[i]
				if gc.getBuildingInfo(iRandomBuilding).isBuildingClassNeededInCity(iLoopBuilding):
					bNeeded = true
					break
			if not bNeeded:
				bFound = true
				lBuildings.remove(iRandomBuilding)
				pCity.setHasRealBuilding(iRandomBuilding, false)
	
	return true
	